/******************************************************************************************
Description: 	Stored Process for three stage modeling in SAS 9.4. 

Creation Date: 	21-Nov-2016

Input parameters : 
1. plan_id - Forecast Plan id
2. projectID : Custom Analysis Project id
3. Selection Method for regression analysis
4. Criterion  
5. Holdout percet 
6. Other hidden parameters in the ca_hiddenPrompts standard group

Ouput parameters:
1. status_ind : value 0 indicates success, 1 indicates error

History: Updated to include open SAS code on 1-Aug-2017.
******************************************************************************************/

options mprint mlogic symbolgen;

%macro ddcf_ca_three_stage_model_sas9_4;

%global  job_rc prj_def_no error_msg;

%let job_rc=0;
%let error_msg=;


/*  Execute macro ddcf_initialize_ca_params to load parameters required for custom analysis  */
%ddcf_initialize_ca_params;


%if &indep_num =0 %then %do; 
	%let job_rc=100;
/*	%let error_msg=ERROR: Independent variables are not selected for the forecast.;*/
	%let error_msg=%SYSFUNC(SASMSG(sashelp.ddcfmsg,Independent_variables_are_not_selected_for_the_forecast,  NOQUOTE));
	%goto three_stage_model_end;
%end;  

%let leaf_lvl=%eval(&hierarchy_count_original. - 1);

/* Include open source code for execution */

%if &SYSSCP. eq WIN %then %do;  /* Windows Operating System */
	%include "!sasroot\ddcfsrv\sasmisc\ddcf_ca_3_stg_modeling_code.sas";
%end; 
%else %do;  /* UNIX Operating System */
   	%include "!sasroot/misc/ddcfsrv/ddcf_ca_3_stg_modeling_code.sas";
%end; 

/* Delete tables related to this analysis if these are already present */
%ddcf_ca_del_three_stage_tables;

%ddcf_ca_call_three_stage_mac;

 %ddcf_signoff_statements;

/* Error handling: check the status of each task */
 
data work.job_status;
	attrib task_id format=8.;
	attrib status format=8.;
	stop;
run;
%do i=1 %to &ddcf_total_tasks.;
	%put Job status for three stage modeling;
	%if "&&return&i" ne "" %then %do;
		proc sql noprint;
			insert into work.job_status(task_id,status) values(&i,&&return&i);
		quit;
		%put return value from job &i : &&return&i;
	%end;
%end;
proc sql noprint;
	select count(*) into :retcodes from work.job_status;
quit;
%if &retcodes lt &ddcf_total_tasks %then %let syscc=9999; /*Indicates abrupt ending of a child process*/
proc sql noprint;
	select max(status) into :max_job_status from work.job_status;
quit;
%if &max_job_status gt &syscc %then %let syscc=&max_job_status;
%put Status for parent job: &syscc;


%ddcf_set_rc(&syscc);
%if (&job_rc>4) %then %do;
	%goto three_stage_model_end;
%end;

/* Append the output of subgroups to create final output */
 %ddcf_ca_three_stg_append_subdata;

/* Generate disaggregate forecast if required */
 %ddcf_create_ca_disagg_forecasts;

/* Validate the output if it can be used for downstream processes such as collaboration plan and SAS Inventory Optimization Workbench.
 If the output is valid, set valid_downstream_processes_ind in the table uiart.rpt_project_def is to 1 else set it to 0 */

%ddcf_validate_ca_output(data=_hpf&leaf_lvl..ca_outfor_&prj_def_no.);


%let fcst_leaf_lvl=%eval(&fcst_stat_hier_count_orig. - 1);

data _null_;
  	fcst_stat_hier_str_nm =TRANWRD("&fcst_stat_hier_str_orig_comma.", 'RK', 'NM');
    call symputx('fcst_stat_hier_str_nm',fcst_stat_hier_str_nm);
 run; 

%three_stage_model_end:

%if (&job_rc>4) %then %do;
/*	%let project_status = ERROR;*/
	
	/*Updates the final status in rpt_project_def table*/
/*	proc sql noprint;*/
/*		update uiart.rpt_project_def */
/*		set prj_def_status_cd = "&project_status." */
/*		where kupcase(ktrim(kleft(prj_def_id))) = kupcase(ktrim(kleft("&projectID.")));;*/
/*	quit;*/

	%let status_ind=1;

%end;
%else %do;
	%let status_ind=0;
%end;

%put "status_ind=&status_ind.";


%STPBEGIN;

	ods html file="&resultHTML";

	%if (&job_rc>4) %then %do;

		data work.error;
		ERROR_MSG="&error_msg.";
		run;
		
		proc print data=work.error noobs label;
		var ERROR_MSG;
/*		label ERROR_MSG='Error while executing the stored process';*/
		label ERROR_MSG="%SYSFUNC(SASMSG(sashelp.ddcfmsg,Error_while_executing_the_stored_process,  NOQUOTE))";;
		run;
	%end;
	%else %do;


		title ;
/*		title "&criterion Distribution for &fcst_stat_hier_string_space.";*/
		title "&criterion %SYSFUNC(SASMSG(sashelp.ddcfmsg,Distribution_for,  NOQUOTE)) &fcst_stat_hier_str_nm.";

		ods noproctitle;

		 PROC UNIVARIATE DATA=_hpf&fcst_leaf_lvl..ca_outstat_&prj_def_no.(KEEP=_REGION_ &criterion.) NOPRINT;
         WHERE TRIM(LEFT(_REGION_)) = "FIT";
         VAR &criterion.;
         HISTOGRAM &criterion. /odstitle=title ; 
         RUN;


	%end;

	ods html close;

%STPEND;


%mend;


%ddcf_init;
%ddcf_capture_log(logprefix=custom_analysis_&projectID.,log_id=&plan_id.);

%ddcf_ca_three_stage_model_sas9_4; 

%ddcf_reset_log;


